# -*- Mode: perl; indent-tabs-mode: nil -*-
#
# The contents of this file are subject to the Mozilla Public License
# Version 1.0 (the "License"); you may not use this file except in
# compliance with the License. You may obtain a copy of the License at
# http://www.mozilla.org/MPL/
# 
# Software distributed under the License is distributed on an "AS IS"
# basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
# License for the specific language governing rights and limitations
# under the License.
# 
# The Original Code is the Bugzilla Bug Tracking System.
# 
# The Initial Developer of the Original Code is Netscape Communications
# Corporation. Portions created by Netscape are Copyright (C) 1998
# Netscape Communications Corporation. All Rights Reserved.
# 
# Contributor(s): Terry Weissman <terry@mozilla.org>

use diagnostics;
use strict;

my %knownattachments;

# This routine quoteUrls contains inspirations from the HTML::FromText CPAN
# module by Gareth Rees <garethr@cre.canon.co.uk>.  It has been heavily hacked,
# all that is really recognizable from the original is bits of the regular
# expressions.

sub quoteUrls {
    my $text = shift;		# Take a copy; don't modify in-place.
    return $text unless $text;

    my $base = Param('urlbase');

    my $protocol = join '|',
    qw(afs cid ftp gopher http https mid news nntp prospero telnet wais);

    my %options = ( metachars => 1, @_ );

    my $count = 0;

    # Now, quote any "#" characters so they won't confuse stuff later
    $text =~ s/#/%#/g;

    # Next, find anything that looks like a URL or an email address and
    # pull them out the the text, replacing them with a "##<digits>##
    # marker, and writing them into an array.  All this confusion is
    # necessary so that we don't match on something we've already replaced,
    # which can happen if you do multiple s///g operations.

    my @things;
    while ($text =~ s%((mailto:)?([\w\.\-\+\=]+\@[\w\-]+(?:\.[\w\-]+)+)\b|
                       (\b((?:$protocol):\S+[\w/])))%"##$count##"%exo) {
        my $item = $&;

        $item = value_quote($item);

        if ($item !~ m/^$protocol:/o && $item !~ /^mailto:/) {
            # We must have grabbed this one because it looks like an email
            # address.
            $item = qq{<A HREF="mailto:$item">$item</A>};
        } else {
            $item = qq{<A HREF="$item">$item</A>};
        }

        $things[$count++] = $item;
    }
    while ($text =~ s/\bbug(\s|%\#)*(\d+)/"##$count##"/ei) {
        my $item = $&;
        my $num = $2;
        $item = value_quote($item); # Not really necessary, since we know
                                	# there's no special chars in it.
        $item = qq{<A HREF="${base}show_bug.cgi?id=$num">$item</A>};
        $things[$count++] = $item;
    }
    while ($text =~ s/\*\*\* This bug has been marked as a duplicate of (\d+) \*\*\*/"##$count##"/ei) {
        my $item = $&;
        my $num = $1;
        $item =~ s@\d+@<A HREF="${base}show_bug.cgi?id=$num">$num</A>@;
        $things[$count++] = $item;
    }
    while ($text =~ s/Created an attachment \(id=(\d+)\)/"##$count##"/e) {
        my $item = $&;
        my $num = $1;
        if (exists $knownattachments{$num}) {
            $item = qq{<A HREF="showattachment.cgi?attach_id=$num">$item</A>};
        }
        $things[$count++] = $item;
    }

    $text = value_quote($text);
    $text =~ s/\&#010;/\n/g;

    # Stuff everything back from the array.
    for (my $i=0 ; $i<$count ; $i++) {
        $text =~ s/##$i##/$things[$i]/e;
    }

    # And undo the quoting of "#" characters.
    $text =~ s/%#/#/g;

    return $text;
}

my $loginok = quietly_check_login();
my $id = $::FORM{'id'};
my $userid = "";
my $changeable;

if ($loginok) {
	$userid = DBname_to_id($::COOKIE{'Bugzilla_login'});
} else { 
 	$userid = 0;
}
	
# If this bug has been set to privileged, lets see if we can view it.
if (!CanISee($id, $userid)) {
	print "<H1>Permission denied.</H1>\n";
	if ($loginok) {
		print "Sorry; you do not have the permissions necessary to see\n";
		print "bug $id.\n";
	} else {
		print "Sorry; bug $id can only be viewed when logged\n";
		print "into an account with the appropriate permissions.  To\n";
		print "see this bug, you must first\n";
		print "<a href=\"show_bug.cgi?id=$id&GoAheadAndLogIn=1\">";
		print "log in</a>.";
	}
	exit;
}

my $query = "
select
        bugs.bug_id,
        product,
        version,
        rep_platform,
        op_sys,
        bug_status,
        resolution,
        priority,
        bug_severity,
        Component,
        assigned_to,
        reporter,
        bug_file_loc,
        short_desc,
		target_milestone,
		qa_contact,
		status_whiteboard,
        date_format(creation_ts,'Y-m-d'),
        groupset,
		delta_ts,
		sum(votes.count)
		from bugs left join votes using(bug_id)
		where bugs.bug_id = $id
		group by bugs.bug_id";

SendSQL($query);
my %bug;
my @row;
if (@row = FetchSQLData()) {
    my $count = 0;
    foreach my $field ("bug_id", "product", "version", "rep_platform",
		       		   "op_sys", "bug_status", "resolution", "priority",
		       		   "bug_severity", "component", "assigned_to", "reporter",
		       		   "bug_file_loc", "short_desc", "target_milestone",
                       "qa_contact", "status_whiteboard", "creation_ts",
                       "groupset", "delta_ts", "votes") {
		$bug{$field} = shift @row;
		if (!defined $bug{$field}) {
	    	$bug{$field} = "";
		}
	$count++;
    }
} else {
    print "<H1>Bug not found</H1>\n";
    print "There does not seem to be a bug numbered $id.\n";
    exit;
}

$bug{'assigned_to'} = DBID_to_name($bug{'assigned_to'});
$bug{'reporter'} = DBID_to_name($bug{'reporter'});
$bug{'long_desc'} = GetLongDescription($id);
my $longdesclength = length($bug{'long_desc'});

if (CanIChange($id, $::COOKIE{'Bugzilla_login'}, $bug{'reporter'}, $bug{'assigned_to'})) {
	$changeable = 1;
} else {
	$changeable = 0;
}

GetVersionTable();

#
# These should be read from the database ...
#
my $platform_popup = "";
my $priority_popup = "";
my $sev_popup = "";
my $component_popup = "";
my $component_text = "";
my $cc_element = "";
my $version_popup = "";
my $product_popup = "";

if ($changeable) {
	$platform_popup = "<SELECT NAME=rep_platform>\n" . 
					     make_options(\@::legal_platform, $bug{'rep_platform'}) . 
						 "\n</SELECT>\n";

	$priority_popup = "<SELECT NAME=priority>\n" . 
						 make_options(\@::legal_priority, $bug{'priority'}) . 
						 "\n</SELECT>\n";

	$sev_popup = "<SELECT NAME=bug_severity>\n" .
					make_options(\@::legal_severity, $bug{'bug_severity'}) .
					"\n</SELECT>\n";

	$component_popup = "<B>$bug{'component'}</B><BR>\n" . 
					   make_popup('component', $::components{$bug{'product'}},
    	                             $bug{'component'}, 1, 0);

	# $component_text = "<INPUT NAME=component_text SIZE=20 VALUE=\"\">"; 

	$cc_element = '<INPUT NAME=cc SIZE=60 VALUE="' . ShowCcList($id) . '">';

	$version_popup = "<SELECT NAME=version>\n" .
						make_options($::versions{$bug{'product'}}, $bug{'version'}) .					
						"\n</SELECT>\n";

	$product_popup = "<SELECT NAME=product>\n" .
						make_options(\@::legal_product, $bug{'product'}) .
						"\n</SELECT>\n";

} else {
	$platform_popup = $bug{'rep_platform'}; 
	$priority_popup = $bug{'priority'};
	$sev_popup = $bug{'bug_severity'};
	$component_popup = $bug{'component'};
	$component_text = "";
	$cc_element = ShowCcList($id);
	$version_popup = $bug{'version'};
	$product_popup = $bug{'product'};

}

my $resolution_popup = make_options(\@::legal_resolution_no_dup, $bug{'resolution'}); 

my $URL = $bug{'bug_file_loc'};

if (defined $URL && $URL ne "none" && $URL ne "NULL" && $URL ne "") {
    $URL = "<B><A HREF=\"$URL\">URL:</A></B>";
} else {
    $URL = "<B>URL:</B>";
}

# This will be deprecated as soon as everyone starts to use the new attachment mechanism instead
# but it is here for backwards compatibility
my $email = "";
if ( -d "data/maildir/$bug{'bug_id'}" ) {
    my $file;
	$email .= "<U><B>$bug{'bug_id'} Mail</B></U><BR>\n";
	opendir (DIR, "data/maildir/$bug{'bug_id'}") || die "Cannot open maildir!: $!";
    while ($file = readdir(DIR)) {
        if ($file ne "." and $file ne "..") {
            $email .= "<A HREF=\"data/maildir/$bug{'bug_id'}/$file\">Email $file</A><BR>\n";
        }
    }
    close(DIR);
}

# Ok, this is a hack but since I moved the description up further it was necessary. The proper way
# would be to get all the dataand place into a dataset to be used now and later on instead of
# querying the database twice.
SendSQL("select attach_id from attachments where bug_id = $id");
while (MoreSQLData()) {
    my $attachid = (FetchOneColumn());
    $knownattachments{$attachid} = 1;
}

print "
<FORM NAME=changeform METHOD=POST ACTION=\"process_bug.cgi\">
<INPUT TYPE=HIDDEN NAME=\"delta_ts\" VALUE=\"$bug{'delta_ts'}\">
<INPUT TYPE=HIDDEN NAME=\"longdesclength\" VALUE=\"$longdesclength\">
<INPUT TYPE=HIDDEN NAME=\"id\" VALUE=$id>
<INPUT TYPE=HIDDEN NAME=\"was_assigned_to\" VALUE=\"$bug{'assigned_to'}\">

<TABLE CELLPADDING=3 CELLSPACING=0 BORDER=1 WIDTH=800>
<TR BGCOLOR=\"#CFCFCF\">
	<TH ALIGN=center>Product - Version</TH>
	<TH ALIGN=center>Component</TH>
	<TH ALIGN=center>Status</TH>
	<TH ALIGN=left>Short Summary</TH>
</TR><TR BGCOLOR=\"ECECEC\">
	<TD ALIGN=center>$bug{'product'} - $bug{'version'}</TD>
	<TD ALIGN=center>$bug{'component'}</TD>
	<TD ALIGN=center>$bug{'bug_status'}</TD>
	<TD ALIGN=left>$bug{'short_desc'}</TD>
</TR>
</TABLE>

<P>
<TABLE WIDTH=800 CELLSPACING=0 CELLPADDING=3 BORDER=1>
<TR BGCOLOR=\"CFCFCF\">
	<TD ALIGN=left><B>Opened by</B> <A HREF=\"mailto:$bug{'reporter'}\">$bug{'reporter'}</A> 
	<B>on</B> &nbsp;$bug{'creation_ts'}&nbsp;&nbsp; <B>Long Description</B></TD>
<TR BGCOLOR=\"#ECECEC\">
	<TD ALIGN=left><PRE>";
print quoteUrls($bug{'long_desc'}, email=>1, urls=>1);
print "	
	</PRE></TD>
</TR>
</TABLE>

<P>
<TABLE WIDTH=800 CELLSPACING=0 CELLPADDING=4 BORDER=1>
<TR BGCOLOR=\"#CFCFCF\">
	<TH ALIGN=left><B>Additional Information</B></TH>
</TR><TR BGCOLOR=\"#ECECEC\">
	<TD>
	<TABLE BORDER=0 WIDTH=\"100%\">
	<TR>
		<TD ALIGN=RIGHT><B>Bug#:</B></TD><TD><A HREF=\"show_bug.cgi?id=$bug{'bug_id'}\">$bug{'bug_id'}</A></TD>
		<TD ALIGN=RIGHT><B><A HREF=\"bug_status.cgi#rep_platform\">Platform:</A></B></TD>
		<TD>$platform_popup</TD>
		<TD ALIGN=RIGHT><B>Version:</B></TD>
		<TD>$version_popup</TD>
	</TR><TR>
		<TD ALIGN=RIGHT><B>Product:</B></TD>
		<TD>$product_popup</TD>
		<TD ALIGN=RIGHT><B>Reporter:</B></TD><TD>$bug{'reporter'}</TD>
		<TD ALIGN=RIGHT ROWSPAN=3><B><A HREF=\"describecomponents.cgi?product=" .
		url_quote($bug{'product'}) . "\">Component:</A></TD>
		<TD ROWSPAN=3>$component_popup</TD>
	</TR><TR>
		<TD ALIGN=RIGHT><B><A HREF=\"bug_status.cgi\">Status:</A></B></TD>
		<TD>$bug{'bug_status'}</TD>
		<TD ALIGN=RIGHT><B><A HREF=\"bug_status.cgi#priority\">Priority:</A></B></TD>
		<TD>$priority_popup</TD>
	</TR><TR>
		<TD ALIGN=RIGHT><B><A HREF=\"bug_status.cgi\">Resolution:</A></B></TD>
		<TD>$bug{'resolution'}</TD>
		<TD ALIGN=RIGHT><B><A HREF=\"bug_status.cgi#severity\">Severity:</A></B></TD>
		<TD>$sev_popup</TD>
	</TR><TR>
		<TD ALIGN=RIGHT><B><A HREF=\"bug_status.cgi#assigned_to\">Assigned To:</A></B></TD>
		<TD>$bug{'assigned_to'}</TD>
		<TD ALIGN=center>$email&nbsp;</TD>
		<TD ALIGN=right COLSPAN=2><B><A HREF=\"bug_status.cgi#component\">Component Text:</A></B></TD>
		<TD>component_text (coming soon)</TD>
	</TR>
	</TABLE>
	<P>
	<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=3> 
	<TR>
		<TH ALIGN=right><B>Cc:</B></TH>
		<TD ALIGN=left>$cc_element</TD>
	";

if (Param("usetargetmilestone")) {
    my $url = "";
    if (defined $::milestoneurl{$bug{'product'}}) {
        $url = $::milestoneurl{$bug{'product'}};
    }
    if ($url eq "") {
        $url = "notargetmilestone.html";
    }
    if ($bug{'target_milestone'} eq "") {
        $bug{'target_milestone'} = " ";
    }
    push(@::legal_target_milestone, " ");
    print "
	</TR><TR>
		<TH ALIGN=RIGHT><A href=\"$url\"><B>Target Milestone:</B></A></TH>
		<TD ALIGN=left><SELECT NAME=target_milestone>" .
    	make_options(\@::legal_target_milestone,
                 	 $bug{'target_milestone'}) .
        "</SELECT></TD>";
}

print "
	</TR>";

if (Param("useqacontact")) {
    my $name = $bug{'qa_contact'} > 0 ? DBID_to_name($bug{'qa_contact'}) : "";
    print "
	<TR>
		<TH ALIGN=right><B>QA Contact:</B></TH>
	";
	if($changeable) {
		print "
		<TD ALIGN=left><INPUT NAME=qa_contact VALUE=\"" .
		value_quote($name) .
		"\" SIZE=60></TD>
	</TR>";
	} else {
	print "
		<TD ALIGN=left>$name</TD>
	</TR>";
	}
}

if ($changeable) {
	print "
	<TR>
		<TH ALIGN=right>$URL</TH>
		<TD COLSPAN=2 ALIGN=left><INPUT NAME=bug_file_loc VALUE=\"$bug{'bug_file_loc'}\" SIZE=60></TD>
	</TR><TR>
		<TH ALIGN=right><B>Summary:</B></TH>
		<TD COLSPAN=2 ALIGN=left><INPUT NAME=short_desc VALUE=\"" .
		value_quote($bug{'short_desc'}) .
		"\" SIZE=60></TD>
	</TR>";
} else {
   print "
    <TR>
        <TH ALIGN=right>$URL</TH>
        <TD ALIGN=left>$bug{'bug_file_loc'}</TD>
    </TR><TR>
        <TH ALIGN=right><B>Summary:</B></TH>
        <TD ALIGN=left>$bug{'short_desc'}</TD>
    </TR>";
}

if (Param("usestatuswhiteboard")) {
	if ($changeable) {
    	print "
	<TR>
		<TH ALIGN=right><B>Status Whiteboard:</B></TH>
		<TD ALIGN=left><INPUT NAME=status_whiteboard VALUE=\"" .
		value_quote($bug{'status_whiteboard'}) .
		"\" SIZE=60></TD>
	</TR>";
	} else {
       print "
    <TR>
        <TH ALIGN=right><B>Status Whiteboard:</B></TH>
        <TD ALIGN=left>$bug{'status_whiteboard'}</TD>
    </TR>";
	}
}

print "
	<TR>
		<TD ALIGN=right VALIGN=top><B>Attachments:</B></TD>
		<TD ALIGN=left>
		<TABLE BORDER=0 CELLSPACING=0 CELLPADDING=3>
";

SendSQL("select attach_id, creation_ts, description, filename, mimetype from attachments where bug_id = $id");
while (MoreSQLData()) {
    my ($attachid, $date, $desc, $filename, $mimetype) = (FetchSQLData());
    if ($date =~ /^(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)(\d\d)$/) {
        $date = "$3/$4/$2 $5:$6";
    }
    my $link = "showattachment.cgi?attach_id=$attachid";
    if ($filename =~ /^.*[\/\\](.*)$/) {
	   $filename = $1;
    }
    $desc = value_quote($desc);
	print qq{
		<TR>
			<TD ALIGN=left><A HREF="$link">$filename</A></TD>
			<TD ALIGN=left>$date</TD>
			<TD ALIGN=left>$desc</TD>
			<TD ALIGN=left>$mimetype</TD>
		</TR>
};
	# See earlier not about attachments and the long description 
    # $knownattachments{$attachid} = 1;
}

if ($changeable) {
	print "
		<TR>
			<TD COLSPAN=3><a href=\"createattachment.cgi?id=$id\">Create a new attachment</A> (proposed patch, testcase, etc.)</TD>
		</TR>
";
}

print"
		</TABLE>
		</TD>
	</TR>
	</TABLE>
";

sub EmitDependList {
    my ($desc, $myfield, $targetfield) = (@_);
    print "<th align=right>$desc:</th><td>";
    my @list;
    SendSQL("select dependencies.$targetfield, bugs.bug_status " . 
 			"from dependencies, bugs " . 
 			"where dependencies.$myfield = $id " .
   			"and bugs.bug_id = dependencies.$targetfield " . 
 			"order by dependencies.$targetfield");
    while (MoreSQLData()) {
        my ($i, $stat) = (FetchSQLData());
        push(@list, $i);
        my $opened = ($stat eq "NEW" || $stat eq "ASSIGNED" ||
                      $stat eq "REOPENED");
        if (!$opened) {
            print "<strike>";
        }
        print qq{<a href="show_bug.cgi?id=$i">$i</a>};
        if (!$opened) {
            print "</strike>";
        }
        print " ";
    }
	if ($changeable) {
		print "</td><td><input name=$targetfield value=\"" .
			  join(',', @list) . "\"></td>\n";
	} else {
		print "<TD>&nbsp;</TD>\n";
	}
}

if (Param("usedependencies")) {
    print "
	<P>
	<TABLE CELLSPACING=3 CELLPADDING=0>
	<TR>
	";
    EmitDependList("Bugs that bug $id depends on", "blocked", "dependson");
    print qq{
		<td rowspan=2><a href="showdependencytree.cgi?id=$id">Show dependency tree</a>
};
    if (Param("webdotbase") ne "") {
        print qq{
		<br><a href="showdependencygraph.cgi?id=$id">Show dependency graph</a>
};
    }
    print "
		</td>
	</tr><tr>
	";
    EmitDependList("Bugs depending on bug $id", "dependson", "blocked");
    print "
	</tr>
	</table>
	";
}

if ($::prodmaxvotes{$bug{'product'}}) {
    print qq{
<table WIDTH="100%"><tr>
<th><a href="votehelp.html">Votes</a> for bug $id:</th><td>
<a href="showvotes.cgi?bug_id=$id">$bug{'votes'}</a>
&nbsp;&nbsp;&nbsp;<a href="showvotes.cgi?voteon=$id">Vote for this bug</a>
</td></tr></table>
};
}

print "
	</TD>
</TR><TR BGCOLOR=\"#CFCFCF\">
	<TH ALIGN=left><B>Additional Comments</B></TH>
</TR><TR BGCOLOR=\"#ECECEC\">
	<TD ALIGN=center><TEXTAREA WRAP=HARD NAME=comment ROWS=10 COLS=80></TEXTAREA></TD>
</TR>
</TABLE>
";

if ($changeable) {
	print "
<P>
<TABLE CELLPADDING=3 WIDTH=800 CELLSPACING=0>
<TR>
    <TD ALIGN=left VALIGN=top>
    <TABLE CELLSPACING=0 CELLPADDING=3 BORDER=1 WIDTH=\"100%\"> 
    <TR BGCOLOR=\"#CFCFCF\">
        <TD ALIGN=center><B>Change State or Resolution</B>
		<BR>(Allowed only by reporter and privileged members)</TD>
    </TR><TR BGCOLOR=\"ECECEC\">
        <TD ALIGN=left>
		<INPUT TYPE=radio NAME=knob VALUE=none CHECKED>
         	Leave as <b>$bug{'bug_status'} $bug{'resolution'}</b><br>
";

# knum is which knob number we're generating, in javascript terms.

	my $knum = 1;

	my $status = $bug{'bug_status'};

	if ($status eq "NEW" || $status eq "ASSIGNED" || $status eq "REOPENED") {
    	if ($status ne "ASSIGNED") {
        	print "<INPUT TYPE=radio NAME=knob VALUE=accept>";
			print "Accept bug (change status to <b>ASSIGNED</b>)<br>";
        	$knum++;
    	}
    	if ($bug{'resolution'} ne "") {
        	print "<INPUT TYPE=radio NAME=knob VALUE=clearresolution>\n";
        	print "Clear the resolution (remove the current resolution of\n";
        	print "<b>$bug{'resolution'}</b>)<br>\n";
        	$knum++;
    	}
    	print "<INPUT TYPE=radio NAME=knob VALUE=resolve>
        	   Resolve bug, changing <A HREF=\"bug_status.cgi\">resolution</A> to
        	   <SELECT NAME=resolution
               ONCHANGE=\"document.changeform.knob\[$knum\].checked=true\">
          	   $resolution_popup</SELECT><br>\n";
    	$knum++;
    	print "<INPUT TYPE=radio NAME=knob VALUE=duplicate>
        	  Resolve bug, mark it as duplicate of bug # 
        	  <INPUT NAME=dup_id SIZE=6 ONCHANGE=\"document.changeform.knob\[$knum\].checked=true\"><br>\n";
    	$knum++;
    	my $assign_element = "<INPUT NAME=\"assigned_to\" SIZE=32 ONCHANGE=\"document.changeform.knob\[$knum\].checked=true\" VALUE=\"$bug{'assigned_to'}\">";

    	print "<INPUT TYPE=radio NAME=knob VALUE=reassign> 
        	  <A HREF=\"bug_status.cgi#assigned_to\">Reassign</A> bug to
          	  $assign_element
        	  <br>\n";
    	$knum++;
    	print "<INPUT TYPE=radio NAME=knob VALUE=reassignbycomponent>
              Reassign bug to owner of selected component<br>\n";
    	$knum++;
	} else {
    	print "<INPUT TYPE=radio NAME=knob VALUE=reopen> Reopen bug<br>\n";
    	$knum++;
    	if ($status eq "RESOLVED") {
        	print "<INPUT TYPE=radio NAME=knob VALUE=verify>
        	      Mark bug as <b>VERIFIED</b><br>\n";
        	$knum++;
    	}
    	if ($status ne "CLOSED") {
        	print "<INPUT TYPE=radio NAME=knob VALUE=close>
        	Mark bug as <b>CLOSED</b><br>\n";
        	$knum++;
    	}
	}

	print "
		</TD>
	</TR>
	</TABLE>
	</TD>
";

	# Find out which groups we are a member of and form radio buttons
	SendSQL("select user_group.groupid, groups.description, groups.isbuggroup " .
    	    "from user_group, groups " .
        	"where user_group.groupid = groups.groupid " .
        	"and user_group.userid = $userid order by groups.groupid");
	my %grouplist;
	my @buggrouplist;
	my @row;
	my $flag = 0;
	while (@row = FetchSQLData()) {
    	if ($row[2] == 0) {
        	next;
    	}
    	$grouplist{$row[0]} = $row[1]; 
    	$flag = 1;
	}

	if ($flag) {
    	print "
	<TD ALIGN=right VALIGN=top>
	<TABLE CELLSPACING=0 CELLPADDING=3 BORDER=1>
	<TR BGCOLOR=\"#CFCFCF\">
		<TD ALIGN=center><B>Groups that can see this bug.</B><BR> 
			(If all <B>Unchecked</B> then same as <B>Everyone</B>)</TD>
	</TR><TR BGCOLOR=\"ECECEC\">
		<TD ALIGN=left>
";

		SendSQL("select groupid from bug_group where bugid = $id");
		while (@row = FetchSQLData()) {
    		push (@buggrouplist, $row[0]);
		}

		foreach my $group (keys %grouplist) {
    		my $checked = lsearch(\@buggrouplist, $group) >= 0 ? "CHECKED" : "";
    		print "<INPUT TYPE=checkbox NAME=group-$group $checked VALUE=1>\n";
    		print "<B>$grouplist{$group}</B> can only see this bug.<BR>\n";
		}

		print "
		</TD>
	</TR>
	</TABLE>
	</TD>
";
	}

	print "
</TR>
</TABLE>
";
}

# added <INPUT TYPE=hidden NAME=reporter VALUE=$bug{'reporter'}> to get the necessary
# redhat changes in process_bug.cgi to work which disallows all users to make changes 
print "
<P>
<TABLE WIDTH=800 CELLPADDING=3  CELLSPACING=0>
<TR>";

if ($changeable) {
	print "
	<TD ALIGN=left VALIGN=top>
	<TABLE CELLSPACING=0 CELLPADDING=3 BORDER=1>
	<TR BGCOLOR=\"#CFCFCF\">
		<TD ALIGN=center><B>Private Changes</B><BR>
		(Reporter, assigned, or privileged members only)</TD>
	</TR><TR BGCOLOR=\"#ECECEC\">
		<TD ALIGN=center>
		<TABLE> 
		<TR>
			<TD ALIGN=center VALIGN=top>	
			<INPUT TYPE=\"submit\" VALUE=\"Save Changes\">
			</TD><TD ALIGN=center VALIGN=top>
			<INPUT TYPE=\"reset\" VALUE=\"Reset\">
			<INPUT TYPE=hidden name=form_name VALUE=process_bug>
			<INPUT TYPE=hidden NAME=reporter VALUE=\"$bug{'reporter'}\">
			</FORM>
			</TD>
";

	if (UserInGroup("errata")) {
		print "
		<TD ALIGN=center VALIGN=top>
		<FORM ACTION=\"http://porkchop.redhat.com/bugzilla/createerrata.cgi\">
		<INPUT TYPE=hidden NAME=product VALUE=\"$bug{'product'}\">
		<INPUT TYPE=hidden NAME=synopsis VALUE=\"$bug{'component'}\">
		<INPUT TYPE=hidden NAME=id_fixed VALUE=\"$::FORM{'id'}\">
		<INPUT TYPE=hidden NAME=\"rep_platform-$bug{'rep_platform'}\" VALUE=\"$bug{'rep_platform'}\">
		<INPUT TYPE=hidden NAME=\"release-$bug{'version'}\" VALUE=\"$bug{'version'}\">
		<INPUT TYPE=submit NAME=action VALUE=\"New Errata\">
		</FORM>
		</TD>
";
	}

	print "
		</TR>
		</TABLE>
		</TD>
	</TR>
	</TABLE>
	</TD>
";
} else {
	print "
	<TD ALIGN=left VALIGN=top>
	<TABLE CELLSPACING=0 CELLPADDING=3 BORDER=1>
	<TR BGCOLOR=\"CFCFCF\">
		<TD ALIGN=center>
		<B>Public Changes</B><BR>(Non-members will need to create account)</TD>
	</TR><TR BGCOLOR=\"#ECECEC\"> 
		<TD>
		<TABLE CELLSPACING=0 CELLPADDING=0 WIDTH=\"100%\">
		<TR>
			<TD ALIGN=center VALIGN=center>
			<INPUT TYPE=submit NAME=add_comment VALUE=\"Add Comment\">
			</FORM></TD>
			<TD ALIGN=center VALIGN=center> 
			<FORM METHOD=post ACTION=process_bug.cgi>
			<INPUT TYPE=hidden NAME=id VALUE=$id>
			<INPUT TYPE=hidden NAME=product VALUE=\"$bug{'product'}\">
			<INPUT TYPE=hidden NAME=version VALUE=\"$bug{'version'}\">
			<INPUT TYPE=hidden NAME=component VALUE=\"$bug{'component'}\">
			<INPUT TYPE=hidden name=knob VALUE=add_cc>
			<INPUT TYPE=submit NAME=add_cc VALUE=\"Add Me to CC List\">
			</FORM></TD>
			<TD ALIGN=center VALIGN=center>
			<FORM METHOD=post ACTION=process_bug.cgi>
			<INPUT TYPE=hidden NAME=id VALUE=$id>
			<INPUT TYPE=hidden NAME=product VALUE=\"$bug{'product'}\">
			<INPUT TYPE=hidden NAME=version VALUE=\"$bug{'version'}\">
			<INPUT TYPE=hidden NAME=component VALUE=\"$bug{'component'}\">
			<INPUT TYPE=hidden name=knob VALUE=rem_cc>
			<INPUT TYPE=submit NAME=add_cc VALUE=\"Remove Me from CC List\">
			</FORM>
			</TD>
		</TR>
		</TABLE>
	</TR>
	</TABLE>
	</TD>
";
} 

print "
</TR>
</TABLE>
";

print "
	</TD>
</TR>
</TABLE>

<P>
<B>
<A HREF=\"show_activity.cgi?id=$id\">[View Bug Activity]</A>&nbsp;&nbsp;
<A HREF=\"long_list.cgi?buglist=$id\">[Format For Printing]</A>
</B>
<P>
<HR ALIGN=\"left\" WIDTH=\"800\">\n";

# To add back option of editing the long description, insert after the above
# long_list.cgi line:
#  <A HREF=\"edit_desc.cgi?id=$id\">Edit Long Description</A>

navigation_header();

1;
